SystemOrganization addCategory: #'Magritte-Deprecated'!

!Symbol methodsFor: '*magritte-deprecated' stamp: 'NickAger 1/30/2012 17:05'!
isDescriptionDefinition
	"Answer wheter the receiver is a method selector following the naming conventions of a description definition."

	^ self isDescriptionSelector
		and: [ self isUnary ]! !

!Symbol methodsFor: '*magritte-deprecated' stamp: 'NickAger 1/30/2012 17:05'!
isDescriptionExtension: aSelector
	"Answer wheter the receiver is a method selector following the naming conventions of a description extension to aSelector."

	^ self isDescriptionSelector
		and: [ self numArgs = 1
		and: [ self beginsWith: aSelector ] ]! !

!Symbol methodsFor: '*magritte-deprecated' stamp: 'NickAger 1/30/2012 17:06'!
isDescriptionSelector
	"Answer wheter the receiver is a method selector following the naming conventions of a  description selector."

	^ self ~= #description
		and: [ self beginsWith: #description ]! !

ProtoObject subclass: #MAProxyObject
	instanceVariableNames: 'realSubject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Deprecated'!

!MAProxyObject commentStamp: 'lr 5/17/2006 15:29' prior: 0!
I represent an abstract proxy object, to be refined by my subclasses.!

MAProxyObject subclass: #MADynamicObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Deprecated'!

!MADynamicObject commentStamp: 'lr 4/10/2007 21:08' prior: 0!
A dynamic object can be used for almost any property within Magritte that is not static but calculated dynamically. This is a shortcut to avoid having to build context sensitive descriptions manually over and over again, however there are a few drawbacks: 

- Some messages sent to this proxy, for example ==#class== and ==#value==, might not get resolved properly.
- Raising an unhandled exception will not always open a debugger on your proxy, because tools are unable to properly work with the invalid object and might even crash your image.!

!MADynamicObject methodsFor: 'accessing' stamp: 'jf 10/28/2009 15:24'!
realSubject
	^ GRPlatform current
		magritteEvaluate: super realSubject
		onUnhandledErrorDo: [ :err | err return: nil ]! !

!MAProxyObject class methodsFor: 'instance creation' stamp: 'jf 9/24/2009 13:08'!
new
	MAError signal: 'Objects of this class require a delegate. Use #on:.'! !

!MAProxyObject class methodsFor: 'instance creation' stamp: 'lr 3/9/2006 11:33'!
on: anObject
	^ self basicNew realSubject: anObject! !

!MAProxyObject methodsFor: 'copying' stamp: 'lr 5/17/2006 15:30'!
copy
	"It doesn't make sense to copy proxies in most cases, the real-subject needs to be looked up and will probably return a new instance on every call anyway."

	^ self! !

!MAProxyObject methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
doesNotUnderstand: aMessage
	^ self realSubject
		perform: aMessage selector
		withArguments: aMessage arguments! !

!MAProxyObject methodsFor: 'magritte-pharo-model' stamp: 'lr 4/10/2007 21:42'!
isMorph
	"Answer ==false==, since I am no morph. Squeak is calling this method after image-startup and might lock if I do not answer to this message."

	^ false! !

!MAProxyObject methodsFor: 'testing' stamp: 'lr 4/10/2007 21:42'!
isNil
	"This method is required to properly return ==true== if the ==realSubject== is ==nil==."

	^ self realSubject isNil! !

!MAProxyObject methodsFor: 'printing' stamp: 'lr 4/10/2007 21:42'!
printOn: aStream
	"Print the receiver on ==aStream== but within square-brackets to show that it is a proxied instance."

	aStream nextPut: $[; print: self realSubject; nextPut: $]! !

!MAProxyObject methodsFor: 'printing' stamp: 'lr 3/9/2006 11:32'!
printString
	^ String streamContents: [ :stream | self printOn: stream ]! !

!MAProxyObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
realSubject
	^ realSubject! !

!MAProxyObject methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:32'!
realSubject: anObject
	realSubject := anObject! !

TestCase subclass: #MADynamicObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Deprecated'!

!MADynamicObjectTest methodsFor: 'tests' stamp: 'jf 9/17/2009 15:42'!
testCalculated
	| object dummy |
	object := [ Time millisecondClockValue ] magritteDynamicObject.
	dummy := object yourself.

	(Delay forMilliseconds: 2) wait.
	self assert: dummy < object yourself! !

!MADynamicObjectTest methodsFor: 'tests' stamp: 'jf 9/17/2009 15:42'!
testCollection
	| object |
	object := [ OrderedCollection with: 1 with: 2 ] magritteDynamicObject.
	self assert: object size = 2.
	self assert: object first = 1.
	self assert: object second = 2.

	object add: 3.
	self assert: object size = 2.
	self assert: object first = 1.
	self assert: object second = 2! !

!MADynamicObjectTest methodsFor: 'tests' stamp: 'jf 9/17/2009 15:43'!
testConstant
	| object |
	object := [ self ] magritteDynamicObject.
	self assert: object = self.

	object := [ 123 ] magritteDynamicObject.
	self assert: object = 123! !

!MADynamicObjectTest methodsFor: 'tests' stamp: 'jf 9/17/2009 15:42'!
testCopy
	| object first second |
	object := [ Time millisecondClockValue ] magritteDynamicObject copy.
	first := object yourself.
	(Delay forMilliseconds: 2) wait.
	second := object yourself.

	self assert: first < second! !

!MADynamicObjectTest methodsFor: 'tests' stamp: 'jf 9/17/2009 15:42'!
testCounter
	| object counter |
	counter := nil.
	object := [
		counter := counter isNil
			ifTrue: [ 1 ]
			ifFalse: [ counter := counter + 1 ] ]
		magritteDynamicObject.

	self assert: object = 1.
	self assert: object yourself = 2.
	self assert: object yourself yourself = 3! !

!MADynamicObjectTest methodsFor: 'tests' stamp: 'jf 9/17/2009 15:42'!
testDynamic
	| object collection |
	collection := nil.
	object := [
		collection isNil
			ifTrue: [ collection := OrderedCollection with: 1 with: 2 ]
			ifFalse: [ collection ] ]
		magritteDynamicObject.

	self assert: object size = 2.
	self assert: object first = 1.
	self assert: object second = 2.

	object add: 3.
	self assert: object size = 3.
	self assert: object first = 1.
	self assert: object second = 2.
	self assert: object third = 3! !

!MADynamicObjectTest methodsFor: 'tests' stamp: 'lr 11/25/2010 23:26'!
testException
	| object |
	object := [ 1 / 0 ] magritteDynamicObject.
	self 
		should: [ object greaseString ]
		raise: ZeroDivide! !

!MADynamicObjectTest methodsFor: 'tests' stamp: 'jf 9/17/2009 15:43'!
testNilOrNotNil
	| object |
	object := [ 1 ] magritteDynamicObject.
	self deny: object isNil.
	self assert: object notNil.

	object := [ nil ] magritteDynamicObject.
	self assert: object isNil.
	self deny: object notNil! !

!BlockContext methodsFor: '*magritte-deprecated' stamp: 'NickAger 1/18/2012 16:20'!
magritteDynamicObject
	"Answer an object that will automatically evaluate the receiver when it receives a message. It will eventually pass the message to the resulting object. Use with caution, for details see *MADynamicObject*."
	
	self greaseDeprecatedApi: 'BlockContext>>#magritteDynamicObject' details: '#magritteDynamicObject is no longer necessary as descriptions are now dynamically created'. 

	^ MADynamicObject on: self! !

!BlockClosure methodsFor: '*magritte-deprecated' stamp: 'NickAger 1/18/2012 16:20'!
magritteDynamicObject
	"Answer an object that will automatically evaluate the receiver when it receives a message. It will eventually pass the message to the resulting object. Use with caution, for details see *MADynamicObject*."
	
	self greaseDeprecatedApi: 'BlockContext>>#magritteDynamicObject' details: '#magritteDynamicObject is no longer necessary as descriptions are now dynamically created'. 

	^ MADynamicObject on: self
! !

!GRPharoPlatform methodsFor: '*magritte-deprecated' stamp: 'jf 9/15/2009 18:15'!
magritteUnregister: anObject forMethodChangeNotificationsUsing: selector
	"Unregister anObject with the system for notification whenever a method is
	added, removed, or modified. anObject should no longer receive notifications
	using the specified selector."
	
	SystemChangeNotifier uniqueInstance
		noMoreNotificationsFor: anObject! !

MABasicObject subclass: #MADescriptionBuilder
	instanceVariableNames: 'cache'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Magritte-Deprecated'!

!MADescriptionBuilder class methodsFor: 'accessing' stamp: 'EstebanLorenzano 1/4/2012 15:48'!
default
	^ Default ifNil: [ Default := MAPragmaBuilder new ]! !

!MADescriptionBuilder class methodsFor: 'accessing' stamp: 'lr 3/27/2006 13:31'!
default: aBuilder
	Default := aBuilder! !

!MADescriptionBuilder class methodsFor: 'building' stamp: 'lr 3/27/2006 13:29'!
for: anObject
	^ self default for: anObject! !

!MADescriptionBuilder class methodsFor: 'initialization' stamp: 'lr 7/5/2009 10:52'!
unload
	self default finalize.
	self default: nil! !

!MADescriptionBuilder methodsFor: 'private' stamp: 'lr 2/14/2007 09:09'!
build: anObject
	self subclassResponsibility! !

!MADescriptionBuilder methodsFor: 'initialization' stamp: 'lr 3/27/2006 13:55'!
finalize
	super finalize.
	self flush; unregister! !

!MADescriptionBuilder methodsFor: 'actions' stamp: 'lr 3/27/2006 13:41'!
flush
	cache := IdentityDictionary new! !

!MADescriptionBuilder methodsFor: 'accessing' stamp: 'lr 9/12/2007 17:26'!
for: anObject
	^ cache at: anObject ifAbsentPut: [ self build: anObject ]! !

!MADescriptionBuilder methodsFor: 'initialization' stamp: 'jf 9/16/2009 13:20'!
initialize
	super initialize.
	self flush; register! !

!MADescriptionBuilder methodsFor: 'events' stamp: 'jf 9/16/2009 17:26'!
modified
	self flush! !

!MADescriptionBuilder methodsFor: 'actions' stamp: 'jf 9/30/2009 00:19'!
register
	GRPlatform current 
		magritteRegister: self
		forMethodChangeNotificationsUsing: #modified! !

!MADescriptionBuilder methodsFor: 'actions' stamp: 'jf 9/30/2009 00:19'!
unregister
	GRPlatform current 
		magritteUnregister: self
		forMethodChangeNotificationsUsing: #modified! !

MADescriptionBuilder subclass: #MANamedBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Deprecated'!

!MANamedBuilder commentStamp: 'lr 4/10/2007 21:09' prior: 0!
I dynamically build container descriptions from class-side methods using a simple naming convention for the selector names:

# The method ==#defaultContainer== is called to retrieve the container instance.
# All the unary methods starting with the selector ==#description== are called and should return a valid description to be added to the container.
# All the keyword messages with one argument having a prefix of a method selected in step 2 will be called with the original description to further refine its definition.!

!MANamedBuilder methodsFor: 'private' stamp: 'NickAger 8/24/2011 13:36'!
build: anObject
	| selectors container |
	selectors := SortedCollection withAll: (anObject class allSelectors select: [ :each | each isDescriptionSelector ]).
	container := self build: anObject for: self containerSelector in: selectors.
	^ (selectors select: [ :each | each isDescriptionDefinition ])
		inject: (cache at: anObject put: container)
		into: [ :result :each |
			self containerSelector = each ifFalse: [
				| description |
				description := self build: anObject for: each in: selectors.
				description isDescription
					ifTrue: [ result add: description ] ].
			result ]! !

!MANamedBuilder methodsFor: 'private' stamp: 'lr 8/23/2008 11:51'!
build: anObject for: aSelector in: aCollection
	^ (aCollection select: [ :each | each isDescriptionExtension: aSelector ])
		inject: (anObject perform: aSelector)
		into: [ :result :each | 
			result isNil 
				ifFalse: [ anObject perform: each with: result ] ]! !

!MANamedBuilder methodsFor: 'configuration' stamp: 'lr 2/14/2007 09:03'!
containerSelector
	^ #descriptionContainer! !

!Object class methodsFor: '*magritte-deprecated' stamp: 'NickAger 1/18/2012 15:51'!
description
	self greaseDeprecatedApi: 'Object class>>#description' details: '#description has been renamed to #magritteDescription as #description is a heavily overloaded term.'. 
	
	^ self magritteDescription! !

!Object methodsFor: '*magritte-deprecated' stamp: 'NickAger 1/18/2012 15:50'!
description
	self greaseDeprecatedApi: 'Object>>#description' details: '#description has been renamed to #magritteDescription as #description is a heavily overloaded term.'. 

	^ self magritteDescription! !
